import prett


class BaseInterface(prett.AttachAbility):
    @property
    def name(self) -> str:
        # noinspection PyUnresolvedReferences
        return self.objectName()

    def set_focus(self):
        # noinspection PyUnresolvedReferences
        return self.setFocus()

    def set_enabled(self, status=True):
        # noinspection PyUnresolvedReferences
        return self.setEnabled(status)

    def set_disabled(self, status=True):
        # noinspection PyUnresolvedReferences
        return self.setEnabled(not status)



from .qt_core import *  # noqa: F403
from prett import SignalSender, connect_with
from .timer import Timer
from .event_loop import EventLoop, wait, later, run_until
assert SignalSender is SignalSender
assert connect_with is connect_with



import prett
from .. import BaseInterface
from .. import QTextEdit
from .. import ui_extension


@ui_extension
class TextEdit(QTextEdit, BaseInterface, prett.WidgetStringInterface):
    class StringItem(prett.WidgetStringItem):
        def __init__(self, parent: 'TextEdit'):
            self.parent = parent

        def get_value(self):
            return self.parent.toPlainText()

        def set_value(self, value):
            value = value or ''
            if value != self.get_value():
                self.parent.setText(value)

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.textChanged.connect(self.string.check_change)



import typing
import numpy as np
import pyqtgraph as pg
from .. import ui_extension
from .. import QFont

pg.setConfigOptions(antialias=True)


@ui_extension
class PlotWidget(pg.PlotWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent, background=(255, 255, 255))

        self.line_pen = pg.mkPen(color=(0, 0, 0), width=3)
        self.axis_pen = pg.mkPen(color=(0, 0, 0), width=1)
        self.labelStyle = {'color': '#000000', 'font-size': '14pt', 'font-family': "微软雅黑"}

        self.getPlotItem().showGrid(x=True, y=True, alpha=0.4)
        self.setMouseEnabled(x=False, y=False)

        font = QFont('Arial', 10)
        axises = ('top', 'bottom', 'left', 'right')
        for axis in axises:
            self.getPlotItem().showAxis(axis)
            self.getPlotItem().getAxis(axis).tickFont = font
            self.getPlotItem().getAxis(axis).setPen(self.axis_pen)

    def plot(self, x=None, y=None):
        """

        :param x: array_like
        :param y: array_like
        :return: None
        """
        self.getPlotItem().plot(x=x, y=y, pen=self.line_pen, symbolBrush=(0, 0, 255), symbolPen='w')

    def append_plot(self, x=None, y=None):
        """

        :param x: array_like
        :param y: array_like
        :return: None
        """
        if x is None or y is None:
            return
        if len(x) != len(y):
            raise Exception('X and Y arrays must be the same shape--got({}, ) and ({}, )'.format(len(x), len(y)))

        old_data = self.get_data()
        if old_data:
            x = np.append(old_data[0], x)
            y = np.append(old_data[1], y)
        # clear old data and repaint
        self.clear()
        self.plot(x=x, y=y)

    def clear(self):
        self.getPlotItem().clear()

    def get_data(self) -> typing.Tuple[np.ndarray, np.ndarray]:
        """

        return last plot data
        """
        if self.getPlotItem().items:
            return self.getPlotItem().items[-1].getData()

    def set_x_label(self, text=None, units=None):
        # noinspection PyArgumentList
        self.getPlotItem().setLabel('bottom', text=text, units=units, **self.labelStyle)

    def set_y_label(self, text=None, units=None):
        # noinspection PyArgumentList
        self.getPlotItem().setLabel('left', text=text, units=units, **self.labelStyle)

    def set_labels(self, text=None, units=None):
        if text is None:
            return
        if len(text) != 2:
            raise ValueError("Label Text Length Must Equal to Two")
        if units is not None:
            if len(text) != len(units):
                raise ValueError("Label Text Length Must Equal to Label Units")
            self.set_x_label(text[0], units[0])
            self.set_y_label(text[1], units[1])
        else:
            self.set_x_label(text=text[0])
            self.set_y_label(text=text[1])



import prett
from .. import ExcitedSignalInterface
from .. import QRadioButton
from .. import ui_extension


@ui_extension
class RatioButton(QRadioButton, ExcitedSignalInterface, prett.WidgetStringInterface):
    def set_excited_signal_connection(self):
        # noinspection PyUnresolvedReferences
        self.clicked.connect(self.excited.emit)

    class StringItem(prett.WidgetStringItem):
        def __init__(self, parent: 'RatioButton'):
            self.parent = parent

        def get_value(self):
            return self.parent.text()

        def set_value(self, value):
            self.parent.setText(value or '')
            self.check_change()



from .. import QPointF, QSizeF


class PointF(QPointF):
    def __add__(self, other):
        if isinstance(other, QSizeF):
            return PointF(self.x() + other.width(), self.y() + other.height())
        else:
            return PointF(self.x() + other.x(), self.y() + other.y())

    def __sub__(self, other):
        if isinstance(other, QSizeF):
            return PointF(self.x() - other.width(), self.y() - other.height())
        else:
            return PointF(self.x() - other.x(), self.y() - other.y())

    def __mul__(self, other):
        return PointF(self.x() * other, self.y() * other)

    def x_add(self, v):
        return PointF(self.x() + v, self.y())

    def x_sub(self, v):
        return PointF(self.x() - v, self.y())

    def y_add(self, v):
        return PointF(self.x(), self.y() + v)

    def y_sub(self, v):
        return PointF(self.x(), self.y() - v)



import prett
from .. import QDate
from .. import QDateEdit
from .. import ui_extension
from .. import BaseInterface


@ui_extension
class DateEdit(QDateEdit, BaseInterface, prett.WidgetStringInterface):
    class StringItem(prett.WidgetStringItem):
        def __init__(self, parent: 'DateEdit'):
            self.parent = parent

        def get_value(self):
            return self.parent.text()

        def set_value(self, value):
            value = value or ''
            if value != self.get_value():
                date = value.split('-')
                if len(date) != 3:
                    raise ValueError('Date format is invalid')
                self.parent.setDate(QDate(int(date[0]), int(date[1]), int(date[2])))

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.dateChanged.connect(self.string.check_change)



from . import PointF
from .. import QPainter
from .. import QSizeF, QRectF
from .. import QFont, QFontMetrics


class SizeF(QSizeF):
    @property
    def w(self):
        return self.width()

    @property
    def h(self):
        return self.height()

    @property
    def size(self):
        return self.width(), self.height()

    def __add__(self, other):
        return super().__add__(other)

    def half_width(self):
        return SizeF(self.width() / 2, self.height())

    def half_height(self):
        return SizeF(self.width(), self.height() / 2)

    def width_add(self, v):
        return SizeF(self.width() + v, self.height())

    def width_sub(self, v):
        return SizeF(self.width() - v, self.height())

    def width_set(self, v):
        return SizeF(v, self.height())

    def height_add(self, v):
        return SizeF(self.width(), self.height() + v)

    def height_sub(self, v):
        return SizeF(self.width(), self.height() - v)

    def height_set(self, v):
        return SizeF(self.width(), v)

    def to_rect(self, p: PointF):
        return QRectF(p, self)

    @staticmethod
    def text_size(text, font: QFont = None):
        if isinstance(font, QPainter):
            font = font.font()
        font_metrics = QFontMetrics(font)

        lines = text.split('\n')
        width = max(map(font_metrics.width, lines))
        height = font_metrics.height() * len(lines)
        return SizeF(width, height)



from .. import QInputDialog
from .. import ui_extension


@ui_extension
class InputDialog(QInputDialog):
    @staticmethod
    def get_double(parent=None,
                   title='Input Double Value',
                   label='Value:',
                   default_value=None,
                   decimals=1,
                   minimum=1,
                   maximum=1e9):
        input_dialog = QInputDialog(parent)
        input_dialog.setInputMode(InputDialog.DoubleInput)
        input_dialog.setWindowTitle(title)
        input_dialog.setLabelText(label)
        input_dialog.setOkButtonText('确定')
        input_dialog.setCancelButtonText('取消')
        input_dialog.setDoubleDecimals(decimals)
        input_dialog.setDoubleRange(minimum, maximum)
        input_dialog.setDoubleValue(default_value or minimum)
        input_dialog.setFont(parent.font())

        return default_value if not input_dialog.exec_() else input_dialog.doubleValue()

    @staticmethod
    def get_int(parent=None, title='Input Int Value', label='Value:', default_value=None, minimum=1, maximum=100):
        input_dialog = QInputDialog(parent)
        input_dialog.setInputMode(InputDialog.IntInput)
        input_dialog.setWindowTitle(title)
        input_dialog.setLabelText(label)
        input_dialog.setOkButtonText('确定')
        input_dialog.setCancelButtonText('取消')
        input_dialog.setIntRange(minimum, maximum)
        input_dialog.setIntValue(default_value or minimum)
        input_dialog.setFont(parent.font())

        return default_value if not input_dialog.exec_() else input_dialog.intValue()



from . import WidgetUiController
from ..gui import Shortcut
from PySide.QtCore import Qt


class DialogUiController(WidgetUiController):
    def __init__(self, parent=None, ui_file=None):
        super().__init__(parent, ui_file)

        Shortcut('ctrl+w', self.w).excited.connect(self.w.close)

    def exec(self):
        return self.w.exec()

    def setWindowMinimizeButtonHint(self):
        self.w.setWindowFlags(Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint)

    @classmethod
    def class_exec(cls, *args, **kwargs):
        return cls(*args, **kwargs).exec()



from .. import QPen
from . import scaling


class Pen(QPen):
    def __init__(self, *args):
        if len(args) >= 2:
            assert isinstance(args[1], int) or isinstance(args[1], float)
            super().__init__(args[0], args[1] * scaling.ratio, *args[2:])
        else:
            super().__init__(*args)

    def setWidth(self, width):
        super().setWidth(int(width * scaling.ratio + 0.5))

    def setWidthF(self, width):
        super().setWidthF(width * scaling.ratio)



import prett
import st
import typing

from .. import ExcitedSignalInterface, RowChangedSignalInterface
from .. import QTableWidget
from .. import Qt, QAbstractItemView, QTableWidgetItem
from .. import ui_extension


@ui_extension
class TableWidget(QTableWidget, ExcitedSignalInterface, prett.WidgetDictInterface, prett.WidgetIndexInterface,
                  prett.WidgetDictListInterface, RowChangedSignalInterface):
    keyPressFunc = None

    def keyPressEvent(self, event):
        if self.keyPressFunc is not None:
            self.keyPressFunc(event)
        event.ignore()

    def set_row_changed_signal_connection(self):
        # noinspection PyUnresolvedReferences
        self.itemClicked.connect(self.row_changed_signal)

    def row_changed_signal(self):
        # noinspection PyUnresolvedReferences
        self.row_changed.emit_if_changed(self.currentRow())

    def set_excited_signal_connection(self):
        # noinspection PyUnresolvedReferences
        self.doubleClicked.connect(st.zero_para(self.excited.emit))

    def set_just_show_mode(self):
        self.auto_resize = True
        self.verticalHeader().hide()
        self.resizeColumnsToContents()
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)

    def set_select_rows_mode(self):
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setStyleSheet("selection-background-color: lightBlue;selection-color: black;")
        self.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.selectAll()
        # noinspection PyUnresolvedReferences
        self.itemClicked.connect(self.cancel_current_select)

    def set_column_hidden(self, header_name):
        header_labels = list(self.horizontalHeaderItem(i).text() for i in range(self.columnCount()))
        if header_name not in header_labels:
            raise ValueError("header_name doesn't match headers label")
        self.hideColumn(header_labels.index(header_name))

    def set_column_show(self, header_name):
        header_labels = list(self.horizontalHeaderItem(i).text() for i in range(self.columnCount()))
        if header_name not in header_labels:
            raise ValueError("header_name doesn't match headers label")
        self.showColumn(header_labels.index(header_name))

    def cancel_current_select(self):
        self.select_row_index = getattr(self, "select_row_index", 0)
        self.select_rows_num = getattr(self, "select_rows_num", 1)
        if self.currentRow() == self.select_row_index and \
                len(self.selectedItems()) == self.select_rows_num * self.columnCount():
            self.clearSelection()
        self.select_rows_num = len(self.selectedIndexes()) / self.columnCount()
        self.select_row_index = self.currentRow()

    def get_selected_list(self) -> typing.List[dict]:
        selected_ids = list(map(lambda x: x.row(), self.selectedIndexes()))
        selected_ids = list(set(selected_ids))
        selected_list = list(filter(lambda x: self.dict_list.value.index(x) in selected_ids, self.dict_list.value))
        return selected_list

    def get_selected_list_without_hidden_col(self) -> typing.List[dict]:
        selected_list = self.get_selected_list()
        header_labels = list(self.horizontalHeaderItem(i).text() for i in range(self.columnCount()))
        # if header_name not in header_labels:
        #     raise ValueError("header_name doesn't match headers label")
        # self.hideColumn(header_labels.index(header_name))
        hidden_columns = []
        for column_name in header_labels:
            if self.isColumnHidden(header_labels.index(column_name)):
                hidden_columns.append(column_name)
        for row in selected_list:
            for hidden_col in hidden_columns:
                del row[hidden_col]
        return selected_list

    def set_headers(self, headers: list):
        self.setColumnCount(len(headers))
        self.setHorizontalHeaderLabels(headers)

    @property
    def auto_resize(self):
        return getattr(self, 'resize', False)

    @auto_resize.setter
    def auto_resize(self, value: bool):
        setattr(self, 'resize', value)

    def get_item_style(self, x, y):
        style_dict = getattr(self, 'item_style', {})
        for xy in [(x, y), (None, y), (x, None), (None, None)]:
            if xy in style_dict.keys():
                return style_dict[xy]
        return None

    # add item style handle func
    def set_item_style(self, x, y, item_func):
        """
        :param x: row num, if it's None, will match all row.
        :param y: col num, if it's None, will match all col.
        :param item_func: func(item: QTableWidgetItem)
        """
        style_dict = getattr(self, 'item_style', {})
        style_dict[(x, y)] = item_func
        setattr(self, 'item_style', style_dict)

    def auto_resize_column_width(self):
        if self.auto_resize:
            self.resizeColumnsToContents()
            col_count = self.columnCount()
            col_width = sum(list([self.columnWidth(i) for i in range(col_count)]))
            if col_width < self.width():
                for i in range(col_count):
                    self.setColumnWidth(i, self.columnWidth(i) / col_width * self.width())

    def resizeEvent(self, event):
        super(TableWidget, self).resizeEvent(event)
        self.auto_resize_column_width()

    class TableWidgetItem:
        def __init__(self, parent: 'TableWidget'):
            self.parent = parent

        @property
        def row_count(self):
            return self.parent.rowCount()

        @property
        def col_count(self):
            return self.parent.columnCount()

        def item_text(self, row, col):
            return self.parent.item(row, col).text()

    class DictItem(TableWidgetItem, prett.WidgetDictItem):
        """get/set current table row text"""

        def get_value(self):
            if self.parent.index.value >= 0:
                current_row = self.parent.currentRow()
                col_count = self.parent.columnCount()
                value = dict()
                for i in range(col_count):
                    value[self.parent.horizontalHeaderItem(i).text()] = self.item_text(current_row, i)
                return value
            return None

        def set_value(self, value):
            assert isinstance(value, dict)
            if len(value) is not self.col_count:
                raise ValueError('Value length must equal to column count')

            texts = self.parent.dict_list.value
            assert isinstance(texts, list)
            if value is None:
                self.parent.index.value = 0
            else:
                for key, v in value.items():
                    value[key] = str(v)
                if value in texts:
                    self.parent.index.value = texts.index(value)
                else:
                    header_labels = list(self.parent.horizontalHeaderItem(i).text() for i in range(self.col_count))
                    self.parent.setRowCount(self.row_count + 1)
                    for i in range(self.col_count):
                        item_text = value[header_labels[i]]
                        if item_text is None:
                            raise ValueError("key value doesn't match headers label")
                        table_item = QTableWidgetItem(item_text)
                        table_item.setTextAlignment(Qt.AlignCenter)
                        item_handle = self.parent.get_item_style(self.row_count - 1, i)
                        if item_handle is not None:
                            item_handle(table_item)
                        self.parent.setItem(self.row_count - 1, i, table_item)
                    self.parent.index.value = self.row_count - 1
                    self.parent.auto_resize_column_width()

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.currentCellChanged.connect(self.check_change)

    class IndexItem(TableWidgetItem, prett.IndexItem):
        """get/set current select row"""

        def get_value(self):
            return self.parent.currentRow()

        def set_value(self, value):
            value = value or 0
            self.parent.selectRow(value)

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.currentCellChanged(self.check_change)

    class DictListItem(TableWidgetItem, prett.DictListItem):
        """ get all table_widget item text"""

        def get_value(self):
            table_texts = []
            for row in range(self.row_count):
                row_dict = dict()
                for col in range(self.col_count):
                    row_dict[self.parent.horizontalHeaderItem(col).text()] = self.item_text(row, col)
                table_texts.append(row_dict)
            return table_texts

        def set_value(self, value: list):
            value = value or []
            assert isinstance(value, list)

            for i in range(self.row_count):
                self.parent.removeRow(0)
            for row_dict in value:
                self.parent.dict.value = row_dict
            self.check_change()



import prett
from .. import QSpinBox
from .. import ui_extension
from .. import BaseInterface


@ui_extension
class SpinBox(QSpinBox, BaseInterface, prett.WidgetStringInterface):
    class StringItem(prett.WidgetStringItem):
        def __init__(self, parent: 'SpinBox'):
            self.parent = parent

        def get_value(self):
            return str(self.parent.value())

        def set_value(self, value):
            if self.get_value() != value:
                self.parent.setValue(int(value or 0))

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.valueChanged[str].connect(self.check_change)



from .. import QWidget
from .. import Painter
from .. import ui_extension
from .. import QCloseEvent, EventLoop
from .. import ClosedSignalInterface, ClassExecInterface, ContainerAbilityInterface


@ui_extension
class Widget(QWidget, ClosedSignalInterface, ClassExecInterface, ContainerAbilityInterface):
    def __init__(self, parent=None, *args):
        if getattr(parent, 'w', None) is not None:
            parent = parent.w
        super().__init__(parent, *args)

    def closeEvent(self, event: QCloseEvent):
        self.closed.emit()
        event.accept()

    def exec(self):
        with EventLoop() as event:
            self.show()
            self.closed.connect(event.quit)

    @property
    def background_color(self):
        return self.create(lambda: None)

    @background_color.setter
    def background_color(self, value):
        self.assign(value)
        self.update()

    @property
    def size(self) -> (int, int):
        return self.width(), self.height()

    def paintEvent(self, *args, **kwargs):
        painter = Painter(self)

        if self.background_color is not None:
            painter.fillRect(self.rect(), self.background_color)
        self.paint(painter)

    def paint(self, painter: Painter):
        pass



# 获取git的tag并保存在RELEASE-VERSION中, 以便每次打包自动识别最新的tag,
# 并且作为version去发布, 如果不存在RELEASE-VERSION文件且无tag, 默认使用"0.0.1"
# 使用前必须在MANIFEST.in中加入
#   include RELEASE-VERSION
#   include version.py

__all__ = ("get_git_version")

import subprocess
import os


def get_git_latest_tag():
    def _minimal_ext_cmd(cmd: str):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd.split(" "), stdout=subprocess.PIPE, env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd("git describe --abbrev=0 --tags")
        git_tag = out.strip().decode('ascii')
        # 去除tag中的v/V
        if str(git_tag).startswith("v") or str(git_tag).startswith("V"):
            git_tag = str(git_tag)[1:]
        if git_tag == "":
            git_tag = None
    except Exception:
        git_tag = None

    return git_tag


def read_release_version():
    try:
        f = open("RELEASE-VERSION", "r")

        try:
            version = f.readlines()[0].strip()
            if version == "":
                return None
            else:
                return version
        finally:
            f.close()
    except Exception:
        return None


def write_release_version(version):
    f = open("RELEASE-VERSION", "w")
    f.write("%s\n" % version)
    f.close()


def get_git_version():
    release_version = read_release_version()
    version = get_git_latest_tag()
    if version is None:
        version = release_version

    # 如果release-version文件没有, 且git没有打tag, 则默认使用"0.0.1"
    if version is None:
        version = "0.0.1"
        # raise ValueError("Cannot find the version number!")

    if version != release_version:
        write_release_version(version)
    return version


if __name__ == "__main__":
    print(get_git_version())



from . import BaseInterface
from .. import SignalSender


class ClosedSignalInterface(BaseInterface):
    @property
    def closed(self) -> SignalSender:
        return self.create(SignalSender)

    @property
    def cannot_closed(self) -> SignalSender:
        return self.create(SignalSender)

    @property
    def can_close(self) -> bool:
        return self.create(lambda: True)

    @can_close.setter
    def can_close(self, value):
        self.assign(value)



ext_classes = set()
deferred_definition = dict()


def ui_extension(cls):
    ext_classes.add(cls)
    return cls


def deferred_define(func):
    deferred_definition[func.__name__] = func
    return func


def run_deferred_function(name, *args, **kwargs):
    return deferred_definition[name](*args, **kwargs)



from .base_interface import BaseInterface
from .closed_signal_interface import ClosedSignalInterface
from .showed_signal_interface import ShowedSignalInterface
from .changed_signal_interface import ChangedSignalInterface
from .excited_signal_interface import ExcitedSignalInterface
from .row_changed_signal_interface import RowChangedSignalInterface
from .ability_interfaces import ContainerAbilityInterface, ClassExecInterface
from .focus_in_signal_interface import FocusInSignalInterface
from .focus_out_signal_interface import FocusOutSignalInterface



from .. import QShortcut
from .. import ui_extension
from .. import QWidget, QKeySequence
from .. import ExcitedSignalInterface


@ui_extension
class Shortcut(QShortcut, ExcitedSignalInterface):
    def __init__(self, key, parent):
        if isinstance(key, str):
            key = QKeySequence(key)
        if not isinstance(parent, QWidget):
            parent = parent.w
        super().__init__(key, parent)

    def set_excited_signal_connection(self):
        # noinspection PyUnresolvedReferences
        self.activated.connect(self.excited.emit)



from . import deferred_define
from . import Widget, QWidget
from . import QMainWindow, QDockWidget
from . import QHBoxLayout, SquareLayout
from . import WidgetController
from . import QSize, QSizeF, QPoint
from . import QPicture, QPixmap
from . import QPrinter, QPainter


@deferred_define
def set_central_widget(self: Widget, widget, del_pre_widget=True):
    if isinstance(widget, WidgetController):
        widget = widget.w
    if not isinstance(widget, QWidget):
        raise TypeError('Only Support Widget or WidgetController')
    widget.setVisible(True)  # ensure widget is visible

    if hasattr(self, 'center_widget'):
        self.layout().removeWidget(self.center_widget)
        self.center_widget.setVisible(False)  # hide pre widget, and widget can reuse
        if del_pre_widget:
            self.center_widget.deleteLater()

    if isinstance(self, QMainWindow):
        self.setCentralWidget(widget)
    elif isinstance(self, QDockWidget):
        self.setWidget(widget)
    elif hasattr(self, 'center_widget'):
        self.layout().addWidget(widget)
    else:
        layout = QHBoxLayout()
        layout.setSpacing(0)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(widget)
        self.setLayout(layout)
    self.center_widget = widget


@deferred_define
def set_square_widget(self: Widget, widget: Widget, spacing=0):
    if isinstance(widget, WidgetController):
        widget = widget.w
    if not isinstance(widget, QWidget):
        raise TypeError('Only Support Widget or WidgetController')

    layout = SquareLayout()
    layout.setSpacing(spacing)
    layout.addWidget(widget)
    self.setLayout(layout)
    self.center_widget = widget


@deferred_define
def set_layout_spacing(self: Widget, spacing):
    layout = self.layout()
    assert isinstance(layout, SquareLayout)
    layout.setSpacing(spacing)
    layout.update()


@deferred_define
def export_to_pdf(self: Widget, filename: str, export_size=QSize(1060, 730)):
    assert isinstance(export_size, QSize)
    w, h = self.size
    if w > h:
        self.resize(export_size.width(), export_size.height())
    else:
        self.resize(export_size.height(), export_size.width())

    p = QPicture()
    painter = QPainter(p)
    self.render(painter, QPoint(0, 0))
    painter.end()

    printer = QPrinter()
    printer.setOutputFormat(QPrinter.PdfFormat)
    printer.setOutputFileName(filename)
    if w > h:
        printer.setOrientation(QPrinter.Landscape)
    if export_size.width() != 1060 or export_size.height() != 730:
        printer.setPageSize(QPrinter.Custom)
        printer.setPaperSize(QSizeF(self.size[1] * 0.8 + 20, self.size[0] * 0.8 + 20), QPrinter.Point)

    painter = QPainter()
    ok = painter.begin(printer)
    if ok:
        painter.drawPicture(0, 0, p)
        ok = painter.end()
    self.resize(w, h)
    return ok


@deferred_define
def export_to_bitmap(self: Widget, filename: str, export_size=QSize(1060, 730)):
    if filename.endswith('pdf'):
        return export_to_pdf(self, filename)
    assert isinstance(export_size, QSize)
    w, h = self.size
    if w > h:
        self.resize(export_size.width(), export_size.height())
    else:
        self.resize(export_size.height(), export_size.width())

    p = QPixmap(*self.size)
    painter = QPainter(p)
    self.render(painter, QPoint(0, 0))
    painter.end()
    ok = p.save(filename)

    self.resize(w, h)
    return ok



from .action import Action
from .combo_box import ComboBox
from .date_edit import DateEdit
from .dialog import Dialog
from .dock_widget import DockWidget
from .double_spin_box import DoubleSpinBox
from .group_box import GroupBox
from .input_dialog import InputDialog
from .label import Label
from .layout import Layout
from .line_edit import LineEdit
from .list_widget import ListWidget
from .main_window import MainWindow
from .plot_widget import PlotWidget
from .push_button import PushButton
from .shortcut import Shortcut
from .spin_box import SpinBox
from .table_widget import TableWidget
from .widget import Widget
from .radio_button import RatioButton
from .text_edit import TextEdit
from .time_edit import TimeEdit
from .table_view import TableView
from .date_time_edit import DateTimeEdit



from . import BaseInterface
from .. import SignalSender


class FocusOutSignalInterface(BaseInterface):
    @property
    def focus_out(self) -> SignalSender:
        return self.create(SignalSender)



from . import BaseInterface
from .. import SignalSender


class ChangedSignalInterface(BaseInterface):
    @property
    def changed(self) -> SignalSender:
        if getattr(self, 'changed_', None) is None:
            setattr(self, 'changed_', SignalSender())
        return getattr(self, 'changed_')



import os
import st
import subprocess
import PySide

pyside_path = os.path.dirname(PySide.__file__)
rcc_path = os.path.join(pyside_path, 'pyside-rcc.exe')
if not os.path.exists(rcc_path):
    rcc_path = 'pyside-rcc.exe'
if not os.path.exists(rcc_path):
    print('PySide Resource Compiler (pyside-rcc.exe) Not Found!')


@st.make_cache
def compile_qrc(filename):
    command = rcc_path + ' -py3 ' + filename
    p = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
    (output, _) = p.communicate()
    p.wait()
    output = output.decode()
    return output


def load_qrc(filename):
    code = compile_qrc(filename)
    st.run(code)



from . import BaseInterface
from .. import SignalSender


class ShowedSignalInterface(BaseInterface):
    @property
    def showed(self) -> SignalSender:
        if getattr(self, 'showed_', None) is None:
            setattr(self, 'showed_', SignalSender())
        return getattr(self, 'showed_')



from . import QTimer, SignalSender


class Timer(QTimer):
    def __init__(self, *args):
        super().__init__(*args)

        self.triggered = SignalSender()
        self.timeout.connect(self.triggered.emit)
        self.timeout = self.triggered



from .. import QAction
from .. import ui_extension
from .. import ExcitedSignalInterface


@ui_extension
class Action(QAction, ExcitedSignalInterface):
    def set_excited_signal_connection(self):
        # noinspection PyUnresolvedReferences
        self.triggered.connect(self.excited.emit)

    def click(self):
        if self.isEnabled():
            self.trigger()



from ..core import *  # noqa: F403
from .qt_gui import *  # noqa: F403
from .ui_extension import ext_classes, ui_extension
from .ui_extension import deferred_define, run_deferred_function

from .interfaces import BaseInterface, ClassExecInterface
from .interfaces import ClosedSignalInterface, ShowedSignalInterface
from .interfaces import ChangedSignalInterface, ExcitedSignalInterface
from .interfaces import RowChangedSignalInterface
from .interfaces import ContainerAbilityInterface
from .interfaces import FocusInSignalInterface
from .interfaces import FocusOutSignalInterface

from .layouts import SquareLayout
from .paint import scaling, PointF, SizeF, Pen, Painter

from .widgets import Widget, Dialog
from .widgets import MainWindow, GroupBox, DockWidget
from .widgets import Label, LineEdit, DateEdit, TextEdit, TimeEdit
from .widgets import ListWidget, ComboBox
from .widgets import SpinBox, DoubleSpinBox
from .widgets import InputDialog
from .widgets import Action, Shortcut, PushButton, RatioButton
from .widgets import TableWidget, TableView
from .widgets import PlotWidget
from .widgets import Layout



from .gui import *  # noqa: F403
from .tools import load_ui, load_qrc, process, auto_generate_cache
from .controller import WidgetController, WidgetUiController, DialogUiController, FileDialogController
from .deferred_function import set_central_widget, set_square_widget



import prett
from .. import QLabel, Qt
from .. import ui_extension
from .. import BaseInterface


@ui_extension
class Label(QLabel, BaseInterface, prett.WidgetStringInterface):
    class StringItem(prett.WidgetStringItem):
        def __init__(self, parent: 'Label'):
            self.parent = parent

        def get_value(self):
            return self.parent.text()

        def set_value(self, value):
            self.parent.setText(value or '')

    def set_clickable_text(self, show_text, call_data, call_func):
        self.setText("<a href=\"{}\">{}</a>".format(call_data, show_text))
        self.setTextFormat(Qt.RichText)
        self.linkActivated.connect(call_func)



from . import SizeF
from .. import QFont


class Scaling:
    def __init__(self):
        base = 800.0  # the width of a character when dpi = 96
        width = SizeF.text_size('Q', QFont('Courier New', 1000)).w
        self.ratio = width / base


scaling = Scaling()



import prett
from .. import QLineEdit
from .. import ui_extension
from .. import FocusInSignalInterface, FocusOutSignalInterface


@ui_extension
class LineEdit(QLineEdit, prett.WidgetStringInterface, FocusInSignalInterface, FocusOutSignalInterface):
    class StringItem(prett.WidgetStringItem):
        def __init__(self, parent: 'LineEdit'):
            self.parent = parent

        def get_value(self):
            return self.parent.text()

        def set_value(self, value):
            value = value or ''
            if value != self.get_value():
                self.parent.setText(value)

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.textChanged.connect(self.string.check_change)

    def focusInEvent(self, e):
        super().focusInEvent(e)
        self.focus_in.emit(self.string.value)

    def focusOutEvent(self, e):
        super().focusOutEvent(e)
        self.focus_out.emit(self.string.value)



from .. import ClosedSignalInterface, ContainerAbilityInterface
from .. import QCloseEvent
from .. import QDialog
from .. import ui_extension


@ui_extension
class Dialog(QDialog, ClosedSignalInterface, ContainerAbilityInterface):
    keyPressFunc = None

    def keyPressEvent(self, event):
        if self.keyPressFunc is not None:
            self.keyPressFunc(event)
        event.ignore()

    def closeEvent(self, event: QCloseEvent):
        if self.can_close:
            self.closed.emit()
            event.accept()
        else:
            self.cannot_closed.emit()
            event.ignore()

    def exec(self, *args):
        super().exec_(*args)

    @property
    def size(self) -> (int, int):
        return self.width(), self.height()



from .. import BaseInterface
from ... import run_deferred_function
from PySide.QtCore import QSize


class ContainerAbilityInterface(BaseInterface):
    def set_central_widget(self, w, **kwargs):
        return run_deferred_function('set_central_widget', self, w, **kwargs)

    def set_square_widget(self, w, spacing=0):
        return run_deferred_function('set_square_widget', self, w, spacing)

    def set_layout_spacing(self, spacing):
        return run_deferred_function('set_layout_spacing', self, spacing)

    def export_to_image(self, filename: str, export_size=QSize(1060, 730)):
        if filename.lower().endswith('.pdf'):
            return self.export_to_pdf(filename, export_size)
        else:
            return self.export_to_bitmap(filename, export_size)

    def export_to_pdf(self, filename, export_size=QSize(1060, 730)):
        return run_deferred_function('export_to_pdf', self, filename, export_size)

    def export_to_bitmap(self, filename, export_size=QSize(1060, 730)):
        return run_deferred_function('export_to_bitmap', self, filename, export_size)



from .widget_controller import WidgetController
from .widget_ui_controller import WidgetUiController
from .dialog_ui_controller import DialogUiController
from .file_dialog_controller import FileDialogController



import prett
import typing
import functools
from .. import ExcitedSignalInterface, RowChangedSignalInterface
from .. import QTableView, QAbstractTableModel
from .. import Qt, QAbstractItemView, QtGui
from .. import ui_extension


@ui_extension
class TableView(QTableView, ExcitedSignalInterface, prett.WidgetDictInterface, prett.WidgetIndexInterface,
                prett.WidgetDictListInterface, RowChangedSignalInterface):
    menuItems = []

    def addMenuItem(self, name, func):
        self.menuItems.append((name, func))

    def contextMenuEvent(self, event):
        if len(self.menuItems) != 0:
            menu = QtGui.QMenu(self)
            for item in self.menuItems:
                name = item[0]
                func = item[1]
                action = QtGui.QAction(name, self)
                pos = event.pos()
                # use lambda will lead to always exec finally func bug, use is ok
                action.triggered.connect(functools.partial(func, pos))
                menu.addAction(action)
            menu.exec_(QtGui.QCursor.pos())

    def set_row_changed_signal_connection(self):
        self.clicked.connect(self.row_changed_signal)

    def row_changed_signal(self):
        if len(self.selectedIndexes()) != 0:
            self.row_changed.emit_if_changed(self.selectedIndexes()[0].row())

    @property
    def model(self):
        if getattr(self, "_model", None) is None:
            self._model = self.Model(self)
            self.setModel(self._model)
        return self._model

    class Model(QAbstractTableModel):
        def __init__(self, parent, *args):
            QAbstractTableModel.__init__(self, parent, *args)
            self.mylist = []
            self.headers = []

        def rowCount(self, parent=None):
            return len(self.mylist)

        def columnCount(self, parent=None):
            return len(self.headers)

        def data(self, index, role):
            if not index.isValid():
                return None
            elif role != Qt.DisplayRole:
                return None
            return self.mylist[index.row()][self.headers[index.column()]]

        def headerData(self, col, orientation, role):
            if orientation == Qt.Horizontal and role == Qt.DisplayRole:
                return self.headers[col]
            return None

        def set_data(self, data):
            self.mylist = data
            # sel.setData(data)

        def set_headers(self, headers):
            self.headers = headers

        def get_by_indexes(self, indexes):
            res = []
            for index in indexes:
                res.append(self.mylist[index])
            return res

        def get_data(self, row, col):
            return self.mylist[row][self.headers[col]]

    def set_headers(self, headers):
        self.model.set_headers(headers)

    def set_data(self, data):
        self.model.set_data(data)
        self.model.reset()
        # fix column from hide to show, but not show in view
        # because origin column size is 0
        self.auto_resize_column_width()

    def get_data(self, row, col):
        data = self.model.get_data(row, col)
        return data

    def set_column_hidden(self, header):
        self.set_column_visible(header, False)

    def set_column_show(self, header):
        self.set_column_visible(header, True)

    def set_column_visible(self, header, is_visible: True):
        if header not in self.model.headers:
            raise ValueError("header_name doesn't match headers label")
        if is_visible:
            self.showColumn(self.model.headers.index(header))
        else:
            self.hideColumn(self.model.headers.index(header))

    def set_just_show_mode(self):
        self.auto_resize = True
        self.verticalHeader().hide()
        self.resizeColumnsToContents()
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)

    def set_select_rows_mode(self):
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setStyleSheet("selection-background-color: lightBlue;selection-color: black;")
        self.setSelectionMode(QAbstractItemView.ExtendedSelection)

    def get_selected_list(self) -> typing.List[dict]:
        selected_ids = list(map(lambda x: x.row(), self.selectedIndexes()))
        selected_ids = list(set(selected_ids))
        selected_list = self.model.get_by_indexes(selected_ids)
        return selected_list

    def get_selected_list_without_hidden_col(self) -> typing.List[dict]:
        selected_list = self.get_selected_list()
        header_labels = self.model.headers
        hidden_columns = []
        for column_name in header_labels:
            if self.isColumnHidden(header_labels.index(column_name)):
                hidden_columns.append(column_name)
        for row in selected_list:
            for hidden_col in hidden_columns:
                if hidden_col in row.keys():
                    del row[hidden_col]
        return selected_list

    @property
    def auto_resize(self):
        return getattr(self, 'resize', False)

    @auto_resize.setter
    def auto_resize(self, value: bool):
        setattr(self, 'resize', value)

    def auto_resize_column_width(self):
        if self.auto_resize:
            self.resizeColumnsToContents()
            col_count = self.col_count
            col_width = sum(list([self.columnWidth(i) for i in range(col_count)]))
            if col_width < self.width():
                for i in range(col_count):
                    self.setColumnWidth(i, self.columnWidth(i) / col_width * self.width())

    def resizeEvent(self, event):
        super(QTableView, self).resizeEvent(event)
        self.auto_resize_column_width()

    @property
    def col_count(self):
        return self.model.columnCount()

    @property
    def row_count(self):
        return self.model.rowCount()

    def select_rows(self, rows):
        self.setSelectionMode(QAbstractItemView.MultiSelection)
        selected_row = list(map(lambda x: x.row(), self.selectedIndexes()))
        for row in rows:
            if row not in selected_row:
                self.selectRow(row)

    def unselect_rows(self, rows):
        self.setSelectionMode(QAbstractItemView.MultiSelection)
        selected_row = list(map(lambda x: x.row(), self.selectedIndexes()))
        for row in rows:
            if row in selected_row:
                self.selectRow(row)



import os
import re
import subprocess
from . import WidgetController
from PySide.QtGui import QFileDialog, QMessageBox, QPushButton, QIcon


class FileDialogController(WidgetController):
    def set_view_icon(self, path):
        self.view_icon_path = path

    def set_finished_icon(self, path):
        self.finished_icon_path = path

    def export_location_dialog(self, export_dir, default_filename, export_format, filename_suffix=None) -> str:
        # export_path = setting.current.saved_export_path.string.value
        # default_filename = project.current.sample_id.string.value
        # if project.current.operator.string.value:
        #     default_filename += '-' + project.current.operator.string.value
        if filename_suffix:
            default_filename += '-' + filename_suffix
        default_filename = re.sub(r'[/\\:*?"<>|]', '-', default_filename)
        default_filename = os.path.join(export_dir, default_filename)

        export_format_filter = "{0} (*.{1})".format(export_format.upper(), export_format)
        filename = QFileDialog.getSaveFileName(self.w, '请选择导出文件夹', default_filename, export_format_filter)
        if isinstance(filename, tuple):
            filename = filename[0]
        assert isinstance(filename, str)
        filename = filename.replace('/', '\\')
        return filename

    def export_finished_dialog(self, export_ok, filename):
        if export_ok:
            msg_box = QMessageBox(self.w)
            msg_box.setWindowTitle('导出成功')
            msg_box.setIcon(QMessageBox.Information)
            msg_box.setText('报告导出成功，路径为：{0}'.format(filename))

            if getattr(self, "view_icon_path", None):
                view_button = QPushButton(QIcon(getattr(self, 'view_icon_path')), '查看', msg_box)
            else:
                view_button = QPushButton('查看', msg_box)
            if getattr(self, "finished_icon_path", None):
                finished_button = QPushButton(QIcon(getattr(self, "finished_icon_path")), '完成', msg_box)
            else:
                finished_button = QPushButton('完成', msg_box)
            msg_box.addButton(view_button, msg_box.NoRole)
            msg_box.addButton(finished_button, msg_box.NoRole)
            msg_box.setDefaultButton(finished_button)
            msg_box.exec_()

            if view_button == msg_box.clickedButton():
                subprocess.Popen('explorer /select,"{0}"'.format(filename))
        else:
            self.warning('导出失败！\n请检查文件权限后重新导出')



from . import Timer
from . import QThread
from . import QEventLoop


class EventLoop:
    def __init__(self, timeout=None, *args):
        self.event = QEventLoop(None, *args)
        self.timeout = timeout

    def quit(self):
        self.event.quit()

    def __enter__(self):
        return self

    def __exit__(self, *_):
        if self.timeout is not None:
            t = Timer()
            t.timeout.connect(self.quit)
            t.start(int(self.timeout * 1000))
        self.event.exec_()


def wait(second=0.01):
    second = max(0.01, second)
    with EventLoop(second):
        pass


later_times = dict()
later_times_cnt = 0


def later(second=0.01, func=None, *args, **kwargs):
    global later_times_cnt

    assert func
    t = Timer()
    t.setSingleShot(True)
    t.timeout.connect(func, *args, **kwargs)

    later_times_cnt += 1
    later_times[later_times_cnt] = t

    def del_current_timer(cnt):
        del later_times[cnt]

    t.timeout.connect(del_current_timer, later_times_cnt)

    t.start(int(second * 1000))


def run_until(func):
    class Thread(QThread):
        def run(self):
            func()

    with EventLoop() as loop:
        thread = Thread()
        # noinspection PyUnresolvedReferences
        thread.finished.connect(loop.quit)
        thread.start()



import st
from .. import SignalSender


def process(target, args=(), finished=None):
    if finished is not None:
        finished_signal = SignalSender()
        finished_signal.connect(finished)
        st.process(target, args=args, finished=finished_signal.emit)
    else:
        st.process(target, args=args)



import st
import prett
from .. import QComboBox
from .. import ui_extension
from .. import BaseInterface


@ui_extension
class ComboBox(QComboBox, BaseInterface, prett.WidgetStringInterface, prett.WidgetIndexInterface,
               prett.WidgetStringListInterface):
    class ComboBoxItem:
        def __init__(self, parent: 'ComboBox'):
            self.parent = parent

        @property
        def count(self):
            return self.parent.count()

        def add_strings(self, *text):
            self.parent.addItems(text)

    class StringItem(ComboBoxItem, prett.WidgetStringItem):
        def get_value(self):
            return self.parent.currentText()

        def set_value(self, value=None):
            texts = self.parent.string_list.value
            assert isinstance(texts, list)

            if value is None:
                self.parent.index.value = 0
            elif value in texts:
                self.parent.index.value = texts.index(value)
            else:
                self.add_strings(value)
                self.parent.index.value = self.count - 1

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.currentIndexChanged[str].connect(self.check_change)

    class IndexItem(ComboBoxItem, prett.IndexItem):
        def get_value(self):
            return self.parent.currentIndex()

        def set_value(self, value):
            if value is None or value >= self.count:
                value = 0
            self.parent.setCurrentIndex(value)

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.currentIndexChanged[int].connect(self.check_change)

    class StringsItem(ComboBoxItem, prett.StringsItem):
        def get_value(self):
            return st.foreach(self.parent.itemText, range(self.count))

        def set_value(self, value):
            value = value or []

            old_count = self.count
            new_count = len(value)
            for i in range(min(old_count, new_count)):
                self.parent.setItemText(i, value[i])
            for index in reversed(range(new_count, old_count)):
                self.parent.removeItem(index)

            self.parent.addItems(value[old_count:new_count])

            self.parent.index.emit_changed()
            self.parent.string.emit_changed()

            self.string_list.check_change()



from . import BaseInterface
from .. import SignalSender


class ExcitedSignalInterface(BaseInterface):
    @property
    def excited(self) -> SignalSender:
        return self.create(SignalSender, finished_with=self.set_excited_signal_connection)

    def set_excited_signal_connection(self):
        pass



from . import PointF, SizeF, scaling
from .. import QPainter, QColor, QPen, QRectF, Qt


class Painter(QPainter):
    default_background_color = QColor(222, 222, 222, 196)

    def draw_text(self,
                  text_rect: QRectF,
                  text: str,
                  color: QColor = None,
                  background_color: QColor = None,
                  background_round_size: int = 2):
        current_pen = self.pen()

        self.setPen(Qt.transparent)
        if background_color:
            self.setBrush(background_color)
        else:
            self.setBrush(self.default_background_color)
        self.drawRoundedRect(text_rect, background_round_size, background_round_size)
        if color:
            self.setPen(QPen(color))
        else:
            self.setPen(current_pen)
        self.drawText(text_rect, Qt.AlignCenter, text)

        if color:
            self.setPen(current_pen)

    def draw_text_top(self,
                      position: PointF,
                      text: str,
                      margin: int = 3,
                      color: QColor = None,
                      background_color: QColor = None,
                      background_color_round_size: int = 2):
        text_size = SizeF.text_size(text, self)
        text_rect = text_size.to_rect(position - text_size.half_width().height_add(margin * scaling.ratio))

        self.draw_text(text_rect, text, color, background_color, background_color_round_size)

    def draw_text_top_left(self,
                           position: PointF,
                           text: str,
                           margin: int = 3,
                           color: QColor = None,
                           background_color: QColor = None,
                           background_color_round_size: int = 2):
        text_size = SizeF.text_size(text, self)
        text_rect = text_size.to_rect(position - text_size.width_add(margin).height_add(margin * scaling.ratio))

        self.draw_text(text_rect, text, color, background_color, background_color_round_size)

    def draw_text_bottom(self,
                         position: PointF,
                         text: str,
                         margin: int = 3,
                         color: QColor = None,
                         background_color: QColor = None,
                         background_color_round_size: int = 2):
        text_size = SizeF.text_size(text, self)
        text_rect = text_size.to_rect(position - text_size.half_width().height_set(-margin * scaling.ratio))

        self.draw_text(text_rect, text, color, background_color, background_color_round_size)

    def draw_text_left(self,
                       position: PointF,
                       text: str,
                       margin: int = 3,
                       color: QColor = None,
                       background_color: QColor = None,
                       background_color_round_size: int = 2):
        text_size = SizeF.text_size(text, self)
        text_rect = text_size.to_rect(position - text_size.half_height().width_add(margin * scaling.ratio))

        self.draw_text(text_rect, text, color, background_color, background_color_round_size)

    def draw_text_right(self,
                        position: PointF,
                        text: str,
                        margin: int = 3,
                        color: QColor = None,
                        background_color: QColor = None,
                        background_color_round_size: int = 2):
        text_size = SizeF.text_size(text, self)
        text_rect = text_size.to_rect(position - text_size.half_height().width_set(-margin * scaling.ratio))

        self.draw_text(text_rect, text, color, background_color, background_color_round_size)

    def draw_text_bottom_left(self,
                              position: PointF,
                              text: str,
                              margin: int = 3,
                              color: QColor = None,
                              background_color: QColor = None,
                              background_color_round_size: int = 2):
        text_size = SizeF.text_size(text, self)
        text_rect = text_size.to_rect(position - text_size.width_add(margin).height_set(-margin * scaling.ratio))

        self.draw_text(text_rect, text, color, background_color, background_color_round_size)

    def draw_text_bottom_right(self,
                               position: PointF,
                               text: str,
                               margin: int = 3,
                               color: QColor = None,
                               background_color: QColor = None,
                               background_color_round_size: int = 2):
        text_size = SizeF.text_size(text, self)
        text_rect = text_size.to_rect(position.x_add(margin).y_add(margin * scaling.ratio))

        self.draw_text(text_rect, text, color, background_color, background_color_round_size)



from .. import QInputDialog, Qt

from .. import Widget
from .. import QMessageBox


class WidgetController:
    def __init__(self, parent=None, constructor=None):
        assert constructor is not None

        if isinstance(parent, WidgetController):
            parent = parent.w

        self.w = self.__trick__(constructor, parent)

    def question(self, text, title='选择'):
        # noinspection PyCallByClass
        return QMessageBox.question(self.w, title, text, QMessageBox.Yes | QMessageBox.No) == QMessageBox.Yes

    def warning(self, text, title='警告'):
        # noinspection PyCallByClass
        QMessageBox.warning(self.w, title, text)

    def information(self, text, title='提示'):
        # noinspection PyCallByClass
        QMessageBox.information(self.w, title, text)

    def about(self, text, title='关于'):
        # noinspection PyCallByClass
        QMessageBox.about(self.w, title, text)

    def message(self, ok=True, ok_msg='成功', bad_msg='失败'):
        if ok:
            self.information(ok_msg)
        else:
            self.warning(bad_msg)

    def input(self, title, label):
        dialog = QInputDialog()
        (text, bool) = dialog.getText(self.w, title, label)
        return text, bool

    @staticmethod
    def __trick__(constructor, parent) -> Widget:
        return constructor(parent)

    # actions
    def close(self):
        self.w.close()

    def show(self):
        self.w.show()

    def hide(self):
        self.w.hide()



import sys
from . import QDir
from PySide import QtGui
from PySide.QtGui import *  # noqa: F403

# noinspection PyArgumentList
if not QtGui.QApplication.instance():
    if getattr(sys, 'frozen', None):
        # noinspection PyArgumentList
        QtGui.QApplication.addLibraryPath(QDir.currentPath())

    # noinspection PyTypeChecker,PyCallByClass
    QtGui.QApplication.setStyle('cleanlooks')
    QtGui.QApplication([])



"""
QT UI Extension

Author: SF-Zhou
Date: 2016-08-07

See:
https://github.com/sf-zhou/quite
"""

from setuptools import setup, find_packages
from version import get_git_version
import os

# 读取项目下的requirements.txt
req_txt_path = os.path.join(os.path.dirname(__file__), "requirements.txt")
if os.path.exists(req_txt_path):
    with open(req_txt_path) as f:
        req_list = f.readlines()
    req_list = [x.strip() for x in req_list]
else:
    req_list = []


def readme_rst():
    """读取README.rst"""
    with open('README.rst', encoding="utf-8") as f:
        return f.read()


print("use latest tag as version: {}".format(get_git_version()))
print("use requirements.txt as install_requires: {}".format(req_list))

setup(
    name='quite',
    version=get_git_version(),
    description='QT UI Extension',
    long_description=readme_rst(),
    url='https://github.com/KD-Group/quite',
    author='SF-Zhou',
    author_email='sfzhou.scut@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    keywords='qt ui',
    packages=find_packages(exclude=['docs', 'tests', 'examples']),
    include_package_data=True,
    install_requires=req_list,
)



import st

from quite.gui.widgets import DateTimeEdit
from . import WidgetController
from .. import load_ui
from ..gui import Action, SignalSender
from ..gui import DoubleSpinBox, TableWidget, TableView
from ..gui import GroupBox, SpinBox
from ..gui import Label, PushButton, RatioButton
from ..gui import Layout
from ..gui import LineEdit, DateEdit, TextEdit, TimeEdit
from ..gui import ListWidget, ComboBox
from ..gui import Widget, DockWidget, PlotWidget


class WidgetUiController(WidgetController):
    def __init__(self, parent=None, ui_file: str = None):
        assert ui_file is not None

        super().__init__(parent, st.partial_back(load_ui, ui_file))

    def label(self, name=None) -> Label:
        return self.__get_widget__('label', name)

    def button(self, name=None) -> PushButton:
        return self.__get_widget__('button', name)

    def radio_button(self, name=None) -> RatioButton:
        return self.__get_widget__('radio_button', name)

    def edit(self, name=None) -> LineEdit:
        return self.__get_widget__('edit', name)

    def date_edit(self, name=None) -> DateEdit:
        return self.__get_widget__('date_edit', name)

    def date_time_edit(self, name=None) -> DateTimeEdit:
        return self.__get_widget__('date_time_edit', name)

    def time_edit(self, name=None) -> TimeEdit:
        return self.__get_widget__('time', name)

    def text_edit(self, name=None) -> TextEdit:
        return self.__get_widget__('text_edit', name)

    def plot_widget(self, name=None) -> PlotWidget:
        return self.__get_widget__('plot_widget', name)

    def list(self, name=None) -> ListWidget:
        return self.__get_widget__('list', name)

    def combo(self, name=None) -> ComboBox:
        return self.__get_widget__('combo', name)

    def container(self, name=None) -> Widget:
        if name is None:
            return self.w
        return self.__get_widget__('container', name)

    def dock(self, name=None) -> DockWidget:
        return self.__get_widget__('dock', name)

    def group(self, name=None) -> GroupBox:
        return self.__get_widget__('group', name)

    def spin(self, name=None) -> SpinBox:
        return self.__get_widget__('spin', name)

    def table(self, name=None) -> TableWidget:
        return self.__get_widget__('table', name)

    def table_view(self, name=None) -> TableView:
        return self.__get_widget__('table_view', name)

    def double(self, name=None) -> DoubleSpinBox:
        return self.__get_widget__('double', name)

    def action(self, name=None) -> Action:
        obj = self.__get_widget__('action', name)
        if getattr(obj, 'excited', None) is None:
            obj.excited = SignalSender()
            obj.triggered.connect(obj.excited.emit)

            def set_enabled(status=True):
                obj.setEnabled(status)

            def set_disabled(status=True):
                obj.setEnabled(not status)

            def click():
                if obj.isEnabled():
                    obj.trigger()

            obj.click = click
            obj.set_enabled = set_enabled
            obj.set_disabled = set_disabled
        return obj

    def widget(self, name=None) -> Widget:
        return self.container(name).center_widget

    def layout(self, name=None) -> Layout:
        return self.__get_widget__('layout', name)

    def __get_widget__(self, type_name, obj_name):
        return getattr(self.w, type_name + '_' + obj_name, None) or getattr(self.w, obj_name + '_' + type_name)



from .. import EventLoop
from .. import QMainWindow
from .. import QCloseEvent
from .. import ui_extension
from .. import ClassExecInterface
from .. import ClosedSignalInterface
from .. import ContainerAbilityInterface


@ui_extension
class MainWindow(QMainWindow, ClosedSignalInterface, ClassExecInterface, ContainerAbilityInterface):
    def closeEvent(self, event: QCloseEvent):
        if self.can_close:
            self.closed.emit()
            event.accept()
        else:
            self.cannot_closed.emit()
            event.ignore()

    def exec(self):
        with EventLoop() as event:
            self.show()
            self.closed.connect(event.quit)



import st
import prett
from .. import QListWidget
from .. import ui_extension
from .. import ExcitedSignalInterface


@ui_extension
class ListWidget(QListWidget, ExcitedSignalInterface, prett.WidgetStringInterface, prett.WidgetIndexInterface,
                 prett.WidgetStringListInterface):
    def set_excited_signal_connection(self):
        # noinspection PyUnresolvedReferences
        self.doubleClicked.connect(st.zero_para(self.excited.emit))

    class ListWidgetItem:
        def __init__(self, parent: 'ListWidget'):
            self.parent = parent

        @property
        def count(self):
            return self.parent.count()

        def item_text(self, idx):
            return self.parent.item(idx).text()

        def add_text(self, *text):
            self.parent.addItems(text)

    class StringItem(ListWidgetItem, prett.WidgetStringItem):
        def get_value(self):
            if self.parent.index.value >= 0:
                return self.parent.currentItem().text()
            return None

        def set_value(self, value):
            texts = self.parent.string_list.value
            assert isinstance(texts, list)
            if value is None:
                self.parent.index.value = 0
            elif value in texts:
                self.parent.index.value = texts.index(value)
            else:
                self.add_text(value)
                self.parent.index.value = self.count - 1

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.currentTextChanged.connect(self.check_change)

    class IndexItem(ListWidgetItem, prett.IndexItem):
        def get_value(self):
            return self.parent.currentRow()

        def set_value(self, value):
            value = value or 0
            self.parent.setCurrentRow(value)

        def set_changed_connection(self):
            # noinspection PyUnresolvedReferences
            self.parent.currentRowChanged.connect(self.check_change)

    class StringsItem(ListWidgetItem, prett.StringsItem):
        def get_value(self):
            return st.foreach(self.item_text, range(self.count))

        def set_value(self, value):
            value = value or []

            self.parent.clear()
            self.parent.addItems(value)
            self.check_change()



import st
import os
import codecs
from . import load_qrc
from .. import ext_classes
from .. import Widget, scaling
from .. import QByteArray, QBuffer, QIODevice
from xml.etree import ElementTree
from PySide.QtUiTools import QUiLoader


@st.make_cache
def get_ui_content(filename):
    with codecs.open(filename, 'r', 'utf-8') as f:
        text = f.read()
    for cls in ext_classes:
        text = text.replace(cls.__bases__[0].__name__, cls.__name__)
    return text


def process_scaling(ui_content: str, ratio: float) -> str:
    tree = ElementTree.fromstring(ui_content)
    for child in tree.iter('width'):
        if child.text != '16777215':
            child.text = str(int(int(child.text) * ratio))
    for child in tree.iter('height'):
        if child.text != '16777215':
            child.text = str(int(int(child.text) * ratio))
    for child in tree.iter('property'):
        name = child.attrib.get('name', None)
        if name == 'spacing' or name[-6:] == 'Margin' and len(child):
            number = child[0]
            number.text = str(int(int(number.text) * ratio))
    ui_content = ElementTree.tostring(tree, encoding='unicode')
    ui_content = ui_content.replace(' />\n', '/>\n')
    return '<?xml version="1.0" encoding="UTF-8"?>\n' + ui_content + '\n'


def load_ui(parent=None, filename=None) -> Widget:
    assert isinstance(filename, str)

    ui_content = get_ui_content(filename)
    if scaling.ratio != 1.0:
        ui_content = process_scaling(ui_content, scaling.ratio)
    return UiLoader().load(ui_content, parent)


def auto_generate_cache(dir_path: str):
    assert isinstance(dir_path, str)
    if not os.path.isdir(dir_path):
        raise ValueError('Paramter Must be Dir Path')
    for root_dir, _, files in os.walk(os.path.abspath(dir_path)):
        for file in files:
            if os.path.splitext(file)[1] == '.ui':
                load_ui(None, os.path.join(root_dir, file))
            if os.path.splitext(file)[1] == '.qrc':
                load_qrc(os.path.join(root_dir, file))


@st.singleton
class UiLoader:
    def __init__(self):
        self.loader = QUiLoader()
        for cls in ext_classes:
            self.loader.registerCustomWidget(cls)

    def load(self, text, parent=None):
        byte_array = QByteArray(text.encode())
        buffer = QBuffer(byte_array)
        buffer.open(QIODevice.ReadWrite)

        ui = self.loader.load(buffer, parent)
        buffer.close()
        return ui
